package org.rayflood.mikuvat.io;

import static org.rayflood.mikuvat.io.VSQDecoder.getByteArrayString;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.sound.midi.MetaMessage;
import javax.sound.midi.MidiMessage;

public class MidiMeta extends MidiData{
	private static final long serialVersionUID = 315321518296650973L;
	private int type;
	private byte[] data;

	public MidiMeta(MidiMessage mm){
		setMidiEvent(mm);
	}

	public MidiMeta(int type, byte[] data){
		setType(type);
		setData(data);
	}

	public MetaMessage getMidiEvent() throws Exception{
		MetaMessage meta = new MetaMessage();
		meta.setMessage(getType(), getData(), getData().length);
		return meta;
	}

	public void setMidiEvent(MidiMessage mm){
		MetaMessage meta = (MetaMessage)mm;
		setType(meta.getType());
		setData(meta.getData());
	}

	public String toString(){
		List<String> sl = new ArrayList<String>();
		sl.add("Type=" + getType());
		sl.add("Data=" + getByteArrayString(getData()));

		StringBuffer sb = new StringBuffer();
		sb.append("MidiMeta={");
		if(!sl.isEmpty()){
			sb.append(sl.get(0));
			for(int i = 1; i < sl.size(); i++){
				sb.append("," + sl.get(i));
			}
		}
		sb.append("}");
		return sb.toString();
	}

	public boolean isReadied(){
		return true;
	}

	public int compareTo(ImmutableObject obj){
		MidiMeta midiMeta = (MidiMeta)obj;
		return compare(getType(), midiMeta.getType());
	}

	public MidiMeta over(Object obj, boolean fill, boolean has){
		MidiMeta midiMeta = ((MidiMeta)obj).clone();
		setType(midiMeta.getType());
		setData(midiMeta.getData());
		return this;
	}

	public MidiMeta setDefault(){
		return this;
	}

	public static MidiMeta getDefault(){
		MidiMeta midiMeta = new MidiMeta();
		midiMeta.setDefault();
		return midiMeta;
	}

	public MidiMeta delete(){
		setType(0);
		setData(new byte[]{});
		return this;
	}

	public boolean isDeleted(){
		return false;
	}

	public MidiMeta(){
	}

	public int getType(){
		return type;
	}

	public void setType(int type){
		this.type = type;
	}

	public byte[] getData(){
		return data;
	}

	public void setData(byte[] data){
		this.data = data;
	}

	public MidiMeta clone(){
		return (MidiMeta)super.clone();
	}

	public static MidiMeta load(String file) throws Exception{
		return (MidiMeta)ImmutableObject.load(file);
	}

	public static MidiMeta load(File file) throws Exception{
		return (MidiMeta)ImmutableObject.load(file);
	}

	public void store(String file) throws Exception{
		super.store(file);
	}

	public void store(File file) throws Exception{
		super.store(file);
	}

	public static MidiMeta unzip(String file) throws Exception{
		return (MidiMeta)ImmutableObject.unzip(file);
	}

	public static MidiMeta unzip(File file) throws Exception{
		return (MidiMeta)ImmutableObject.unzip(file);
	}

	public void zip(String file) throws Exception{
		super.zip(file);
	}

	public void zip(File file) throws Exception{
		super.zip(file);
	}
}
